/*
    Objet :         Contrôle d'une LED RGB selon la consigne retournee par un capteur ultrason HC-SR04

    Pre-requis :    1 x Carte Arduino (exemple : UNO R3, MEGA)
                    1 x module de capture ultrason HC-SR04
                    1 x LED Rouge (pour les premiers essais, utiliser la LED rouge de la carte Arduino)
                    1 x LED RGB
                    3 x resistance 560 ohm (code : vert, blue, marron)
                    2 x resistance 220 ohm (code : vert, blue, marron)
                    1 x carte d'essais
                    6 x fils male/femelle
                    6 x fils male/male

    Role :          Le capteur ultrason HC-SR04 permet la detection d'un objet et de mesurer la distance
                    entre l'objet detecte et le capteur ultrason HC-SR04.
                    La carte Arduino lit la distance retournee par la capteur et determine la couleur 
                    a affecter a la LED.
                    Ainsi, la couleur emise par la LED sera fonction de la distance lue.

    Description :   Cette fiche fournit les instructions permettant la mise en oeuvre du capteur
                    ultrason HC-04 avec une carte Arduino.

    Caracteristiques du module ultrason : http://www.robotshop.com/eu/fr/module-sonar-hc-sr04.html
                    – Alimentation : 5v.
                    – Consommation en utilisation : 15 mA.
                    – Gamme de distance : 2 cm à 5 m.
                    – Resolution : 0.3 cm.
                    – Angle de mesure : < 15°.

                    - Brochage du module HC-SR04 est le suivant :
                        - VCC : Alimentation +5V DC
                        - Trig : Broche d'entree
                        - Echo : Broche de sortie
                        - GND : Masse d'alimentation

    Caracteristiques de la LED RGB : http://www.robotshop.com/eu/fr/del-trois-couleurs-rgb-5mm-20pk.html
                    - LED Rouge Vert Bleu a anode commune
                    - Diametre : 5 mm
                    - Tension nominale : 5 V
                    - Signaux d'interface :
                        - Anonde commune
                        - Broche Rouge
                        - Broche Verte
                        - Broche Bleue

    Cablage a la carte Arduino :
                    HC-SR05 VCC (5V) --> Arduino 5V
                    HC-SR05 Trig     --> Arduino Pin 13
                    HC-SR05 Echo     --> Arduino Pin 12
                    HC-SR05 GND      --> Arduino GND Pin

                    Anonde commune   --> Arduino 5V
                    Broche Rouge     --> Resistance 220 ohm --> Arduino 9 
                    Broche Verte     --> Resistance 220 ohm --> Arduino 10
                    Broche Bleue     --> Resistance 220 ohm --> Arduino 11

    Principe de controle du module ultrason HC-SR04 :
            Une impulsion de niveau haut (+5v) doit etre envoyee et maintenue pendant au moins 10 µs
            sur la broche "Trig Input" pour que le module mesure de distance.
            En retour, la broche de sortie "Echo" fournit une impulsion + 5v dont la duree est
            proportionnelle a la distance si le module detecte un objet.
            Afin de pouvoir calculer la distance en cm, on utilisera la formule suivante :
                distance = (duree de l'impulsion (en µs) / 58) (cf. Specifications du module HCSR04)
                Ci-dessous, il sera donne une expression permettant de calculer la distance a partir
                de la vitesse de l'onde sonore dans l'air pour une temperature donnee.

    Principe de controle de la LED RGB :
            Le module Arduino configure les broches Rouge Verte et Bleue de la LED RGB a des valeurs
            variant de 0 a 255 sur chacune des broches.
            Recommendations : Filtrer la lumiere emise par LED RGB avec une feuille de papier.

    Prochaine etape de conception :
            Fusionner les fonctions mises en oeuvre pour le capteur SR04 et la LED RGB dans 
            le but de realiser la lampe intelligente.

 */

// Parametres de DEBUG
//#define __DEBUG

#ifdef __DEBUG
#define __print(X)      Serial.print(X)
#define __println(X)    Serial.println(X)
#else
#define __print(X)
#define __println(X)
#endif

// Parametres d'initialisation
// Debit aux interfaces Serial (ie UART)
#define debitBaud     115200

#define FALSE         0
#define TRUE          1

#define __PARTIE_0
//#define __PARTIE_1
//#define __PARTIE_2
//#define __PARTIE_3
//#define __PARTIE_4

// Definition des broches utilisees 

#define ledRougePIN  9  // LED rouge de la carte pour les premiers essais 
#define ledVertPIN   10 // provision : pour l'instant, elle ne sera pas utilisee
#define ledBleuPIN   11 // provision : pour l'instant, elle ne sera pas utilisee

#define echoPIN0     12 // broche dediee a la lecture de la valeur mesuree
#define trigPIN0     13 // broche dediee au declenchement de mesure
#define echoPIN1     7  // broche dediee a la lecture de la valeur mesuree
#define trigPIN1     8  // broche dediee au declenchement de mesure

#ifdef __PARTIE_0           // PARTIE 0
#define PAUSE        200    // pause en ms
#else
#ifdef __PARTIE_1           // PARTIE 1
#define PAUSE        2      // pause en ms
#else
#ifdef __PARTIE_2           // PARTIE 2
#define PAUSE        2      // pause en ms
#else
#ifdef __PARTIE_3           // PARTIE 3
#define PAUSE        2      // pause en ms
#else
#ifdef __PARTIE_4           // PARTIE 4
#define PAUSE        2      // pause en ms
#else                       // PARTIE 5
#define PAUSE        2      // pause en ms
#endif
#endif
#endif
#endif
#endif

// Les lignes suivantes fournissent les moyens pour le calcul de la distance parcourue par l'onde sonore.
// Approxiamtion de la vitesse de l'onde sonore dans l'air a une temperature donnee :
// Vitesse onde sonore (VOS) = 331.5 + 0.6 * [temperature de l'air en degres Celsius]
// Exemple : A 20°C, VOS = 331.5 + 0.6 * 20 = 343.5 (m/s)
// Conversion de la vitesse en centimetres par microseconde :
// VOS = 343.5 * 100 / 1000000 = 0.03435 (cm/us)
// La distance parcourue est donc : Distance = (duree_trajet_aller_retour (us) / 2) * VOS
//      Le facteur 2 de l'expression ci-dessus fait l'hypothese que la duree du trajet aller
//      est egale a la duree du trajet retour.
// Application numerique :
//      Duree du trajet aller-retour = 500 us
//      Distance = 250 * 0.03435 = 8.6 cm
// (si la duree aller-retour = 500us et donc 250us lorsque l'onde sonore frappe l'objet).

// Optimisation pour limiter la perte de performance lors du calcul de vitesse : 
// On posera 1 / VOS = 1 / 0.03435 = 29.1 (us/cm)
// Ainsi l'expression pour calculer la distance devient : Distance = (duree_trajet_aller_retour/2) / (1 / VOS)
// Application numerique : Distance = 250 / 29.1 = 8.6 cm

#define TEMPERATURE_AIR     25      // temperature de l'air en °C
#define VOS                 ( ( 331.5 + 0.6 * TEMPERATURE_AIR ) * 100 ) / 1000000 // en cm/us

const float coeff_multiplicateur = VOS / 2; // optimisation de : (1 / 2) / (1 / VOS);

int maxDistance             = 400;    // Distance maximale (cm)
int minDistance             = 10;     // Distance minimale (cm)
int distance_US0, distance_US1;       // Distances mesurées (cm)

int valeurRouge;
int valeurVert;
int valeurBleu;

long int inByte; 

// Définition des étas de la lampe
typedef enum 
{
    STATE_OFF,
    STATE_ON,
    STATE_PROCESS,
    STATE_MAX
} etats_lampe;

// Définition des évènements déclencheurs
typedef enum 
{
    EVT_OFF,
    EVT_ON,
    EVT_START,
    EVT_MAX
} evts_lampe;

etats_lampe     etat_courant  = STATE_OFF; // Initialisation de la machine à état à STATE_OFF

boolean sensor_0_detected     = false;
boolean sensor_1_detected     = false;

int msecs = 0;

unsigned char cnt;

// ============================================================================
// Nom de la fonction : lectureCmdUtilisateur()
//     Paramètres d'entrée  : NA
//     Paramètres de sortie : NA
//
// Description : Défintion de la fonction des commandes utilisateur 
// ============================================================================
evts_lampe lectureCmdUtilisateur( )
{
    int lecture_echo_0, lecture_echo_1; 
    int distance_0, distance_1;

    unsigned char cmdUtilisateur;

    digitalWrite(trigPIN0, LOW); 
    delayMicroseconds(2);                       // duree de 2us de l'onde sonore emise
    digitalWrite(trigPIN0, HIGH); 
    delayMicroseconds(10);                      // duree de 10us de l'onde sonore emise
    digitalWrite(trigPIN0, LOW); 
    lecture_echo_0 = pulseIn(echoPIN0, HIGH);   // mesure la duree de trajet aller-retour l'onde sonore 

    digitalWrite(trigPIN1, LOW); 
    delayMicroseconds(2);                       // duree de 2us de l'onde sonore emise
    digitalWrite(trigPIN1, HIGH); 
    delayMicroseconds(10);                      // duree de 10us de l'onde sonore emise
    digitalWrite(trigPIN1, LOW); 
    lecture_echo_1 = pulseIn(echoPIN1, HIGH);   // mesure la duree de trajet aller-retour l'onde sonore 

    distance_0 = lecture_echo_0 / 58.2; 
    distance_1 = lecture_echo_1 / 58.2; 
    //distance_0 = (int) ((float)lecture_echo_0 * coeff_multiplicateur);     // en cm
    //distance_1 = (int) ((float)lecture_echo_1 * coeff_multiplicateur);     // en cm

    if ( distance_0 >= maxDistance || distance_0 <= minDistance )
    {
        __println("Mesure_Capteur_0 erronee, objet hors portee ou perdue ou indetectable !");
        distance_0 = maxDistance;
    }
    else
    {
        __print("Distance Capteur_0(cm) : "); 
        __println(distance_0);   // affiche la distance mesuree
    }
    if ( distance_1 >= maxDistance || distance_1 <= minDistance )
    {
        __println("Mesure_Capteur_1 erronee, objet hors portee ou perdue ou indetectable !");
        distance_1 = maxDistance;
    }
    else
    {
        __print("Distance Capteur_1(cm) : "); 
        __println(distance_1);   // affiche la distance mesuree
    }

    distance_US0 = distance_0;
    distance_US1 = distance_1;

    cmdUtilisateur = 3;
    if ( distance_0 < 5 )
    {
        cmdUtilisateur = 0;
    }
    else if ( distance_0 < 15 )
    {
        cmdUtilisateur = 1;
    }
    else if ( distance_0 < 50 )
    {
        cmdUtilisateur = 2;
    }

    __print("CMD - Distance 0 : "); __println(distance_0);
    __print("CMD - USER : "); __println(cmdUtilisateur);
    switch(cmdUtilisateur)
    {
        case 0 :
            __println("CMD - Event value : EVT_OFF = 0");
            return EVT_OFF;
        break;
        
        case 1 :
            __println("CMD - Event value : EVT_ON = 1");
            return EVT_ON;
        break;

        case 2 :
            __println("CMD - Event value : EVT_START = 2");
            return EVT_START;
        break;

        default :
            __println("CMD - Event value : EVT_MAX = 3");
            return EVT_MAX;
        break;
    }
}

// ============================================================================
// Nom de la fonction : Automate()
//     Paramètres d'entrée  : evenement
//     Paramètres de sortie : NA
//
// Description : Défintion de la machine à état de l'automate contenant le code
//               de la lampe à exécuter 
// ============================================================================
void Automate( evts_lampe evenement )
{
    etats_lampe     etat_futur = etat_courant;
    __print("SM - Event value : "); __println(evenement);
    __print("SM - Etat courant : "); __println(etat_courant);
    switch(etat_courant)
    {
        case STATE_OFF :
            switch(evenement)
            {
                case EVT_ON :     // Sur évènement, transition à état futur
                    etat_futur = STATE_ON;
                break;
            }
        break;
        
        case STATE_ON :
            switch(evenement)
            {
                case EVT_OFF :     // Sur évènement, transition à état futur
                    etat_futur = STATE_OFF;
                break;
                
                case EVT_START :     // Sur évènement, transition à état futur
                    etat_futur = STATE_PROCESS;
                break;
            }
        break;

        case STATE_PROCESS :
            switch(evenement)
            {
                case EVT_OFF :     // Sur évènement, transition à état futur
                    etat_futur = STATE_OFF;
                break;
            }
        break;

        default : // Ce cas ne devrait jamais se produire
        break;
    }
    if (etat_futur != etat_courant)
    {
        ActionAlaSortie(etat_courant);
        ActionAlEntree(etat_futur);
        etat_courant = etat_futur;
    }
 
    //if ( evenement != EVT_MAX) ActionTantQueDansEtat( etat_courant );
    if ( etat_courant == etat_futur ) ActionTantQueDansEtat( etat_courant ); 
}

// ============================================================================
// Nom de la fonction : ActionAlEntree()
//     Paramètres d'entrée  : Etat
//     Paramètres de sortie : NA
//
// Description : TODO - Code à exécuter lors de l'entrée dans l'état futur 
// ============================================================================
void ActionAlEntree( int Etat )
{
    switch (Etat)
    {
        case STATE_OFF:
            // TODO : Code à exécuter lors de l'entrée dans l'état futur
            __println("Fonction ActionAlEntree : Action exécutée à l'entrée dans l'état STATE_OFF");
        break;
        case STATE_ON:
            // TODO : Code à exécuter lors de l'entrée dans l'état futur
            __println("Fonction ActionAlEntree : Action exécutée à l'entrée dans l'état STATE_ON");
        break;
        case STATE_PROCESS:
            // TODO : Code à exécuter lors de l'entrée dans l'état futur
            __println("Fonction ActionAlEntree : Action exécutée à l'entrée dans l'état STATE_PROCESS");
        break;
    }
}

// ============================================================================
// Nom de la fonction : ActionTantQueDansEtat()
//     Paramètres d'entrée  : Etat
//     Paramètres de sortie : NA
//
// Description : TODO - Code à exécuter tant que dans l'état courant 
// ============================================================================
void ActionTantQueDansEtat( int Etat)
{
    switch (Etat)
    {
        case STATE_OFF:
            // TODO : Code à exécuter tant que dans l'état courant
            __println("Fonction ActionTantQueDansEtat : Action exécutée dans l'état courant STATE_OFF");
            valeurRouge = 255;
            valeurBleu  = 0;
            valeurVert  = 0;            
            afficherCouleur( valeurRouge, valeurVert, valeurBleu );
            delay(PAUSE * 200);
            // Eteindre toutes les couleurs
            valeurRouge = 0;
            valeurBleu  = 0;
            valeurVert  = 0;            
            afficherCouleur( valeurRouge, valeurVert, valeurBleu );
            delay(PAUSE * 200);
        break;
        case STATE_ON:
            // TODO : Code à exécuter tant que dans l'état courant
            __println("Fonction ActionTantQueDansEtat : Action exécutée dans l'état courant STATE_ON");
            valeurRouge = 0;
            valeurBleu  = 0;
            valeurVert  = 255;            
            afficherCouleur( valeurRouge, valeurVert, valeurBleu );
            delay(PAUSE * 200);
            // Eteindre toutes les couleurs
            valeurRouge = 0;
            valeurBleu  = 0;
            valeurVert  = 0;            
            afficherCouleur( valeurRouge, valeurVert, valeurBleu );
            delay(PAUSE * 200);
            
        break;
        case STATE_PROCESS:
            // TODO : Code à exécuter tant que dans l'état courant
            __println("Fonction ActionTantQueDansEtat : Action exécutée dans l'état courant STATE_PROCESS");
            // Animation par modulation du cycle et de la couleur de la LED selon la distance du l'objet
            animeLampe(distance_US0);
        break;
    }
}

// ============================================================================
// Nom de la fonction : ActionAlaSortie()
//     Paramètres d'entrée  : Etat
//     Paramètres de sortie : NA
//
// Description : TODO - Code à exécuter lors de la sortie de l'état courant 
// ============================================================================
void ActionAlaSortie( int Etat)
{
    switch (Etat)
    {
        case STATE_OFF:
            // TODO : Code à exécuter lors de la sortie de l'état courant
             __println("Fonction ActionAlaSortie : Action exécutée lors de la sortie de l'état courant STATE_OFF");
        break;
        case STATE_ON:
            // TODO : Code à exécuter lors de la sortie de l'état courant
            __println("Fonction ActionAlaSortie : Action exécutée lors de la sortie de l'état courant STATE_ON");
        break;
        case STATE_PROCESS:
            // TODO : Code à exécuter lors de la sortie de l'état courant
            __println("Fonction ActionAlaSortie : Action exécutée lors de la sortie de l'état courant STATE_PROCESS");
        break;
    }
}

// ============================================================================
// Nom de la fonction : afficherCouleur()
//     Paramètres d'entrée  : rouge, vert, bleu
//     Paramètres de sortie : NA
//
// Description : Défintion de la fonction des commandes utilisateur 
// ============================================================================
void afficherCouleur( int rouge, int vert, int bleu )
{
    analogWrite( ledRougePIN, 255 - rouge );
    analogWrite( ledVertPIN,  255 - vert  );
    analogWrite( ledBleuPIN,  255 - bleu  );
}

// ============================================================================
// Nom de la fonction : animeLampe()
//     Paramètres d'entrée  : distance
//     Paramètres de sortie : NA
//
// Description : TODO 
// ============================================================================
void animeLampe( int distance )
{
    if ( distance > 5 )
    {
        // Illuminer la couleur Rouge
        valeurRouge = 255;
	      valeurVert  = 0;
	      valeurBleu  = 0;
    }

    if ( distance > 20 )
    {
        // Illuminer la couleur Verte
        valeurRouge = 0;
	      valeurVert  = 255;
	      valeurBleu  = 0;
    }

    if ( distance > 40 )
    {
        // Illuminer la couleur Bleue
        valeurRouge = 0;
	      valeurVert  = 0;
	      valeurBleu  = 255;
    }

    if ( distance >= maxDistance )
    {
        // Illuminer toutes les couleurs
        valeurRouge = 255;
	      valeurBleu  = 255;
	      valeurVert  = 255;
    }

    afficherCouleur( valeurRouge, valeurVert, valeurBleu );
    delay(distance * PAUSE);

    // Eteindre toutes les couleurs
    valeurRouge = 0;
	  valeurBleu  = 0;
	  valeurVert  = 0;
    afficherCouleur( valeurRouge, valeurVert, valeurBleu );
    delay(distance * PAUSE);
}

// ============================================================================
// Nom de la fonction : setup()
//     Paramètres d'entrée  : NA
//     Paramètres de sortie : NA
//
// Description : Fonction d'initialisation 
// ============================================================================
void setup() 
{ 
    Serial.begin(debitBaud);        // Ouvre le port série 0 pour le transfert de caractères à une débit de "debitBaud"

    sensor_0_detected   = false;
    sensor_1_detected   = false;
    etat_courant        =  STATE_OFF;
    msecs               = 0;
    cnt                 = 0;

    pinMode(ledRougePIN,  OUTPUT);
    pinMode(ledVertPIN,   OUTPUT);
    pinMode(ledBleuPIN,   OUTPUT);

    pinMode(echoPIN0,     INPUT); 
    pinMode(echoPIN1,     INPUT); 
    pinMode(trigPIN0,     OUTPUT); 
    pinMode(trigPIN1,     OUTPUT); 

    digitalWrite (ledRougePIN, HIGH);
    digitalWrite (ledVertPIN,  HIGH);
    digitalWrite (ledBleuPIN,  HIGH);

    digitalWrite(trigPIN0, LOW); 
    digitalWrite(trigPIN1, LOW);
    
    delay( PAUSE );
    Serial.println("FIN INITIALISATION"); 
}

// ============================================================================
// Nom de la fonction : loop()
//     Paramètres d'entrée  : NA
//     Paramètres de sortie : NA
//
// Description : Fonction principale : boucle infinie 
// ============================================================================
void loop() 
{
#ifdef __PARTIE_0
    /* ******************* DEBUT PARTIE 0 ***************** */
    valeurRouge = 255;
    valeurBleu  = 0;
    valeurVert  = 0;

    afficherCouleur( valeurRouge, valeurVert, valeurBleu );
    delay( PAUSE );
  
    valeurRouge = 0;
    valeurBleu  = 0;
    valeurVert  = 255;

    afficherCouleur( valeurRouge, valeurVert, valeurBleu );
    delay( PAUSE );
  
    valeurRouge = 0;
    valeurBleu = 255;
    valeurVert = 0;
   
    afficherCouleur( valeurRouge, valeurVert, valeurBleu );
    delay( PAUSE );
    /* ******************** FIN PARTIE 0 ****************** */
#endif
#ifdef __PARTIE_1
    /* ******************* DEBUT PARTIE 1 ***************** */
    valeurRouge = 255;
    valeurBleu  = 0;
    valeurVert  = 0;
    for( int i = 0 ; i < 255 ; i += 1 )
    {
        valeurVert  += 1;
        valeurRouge -= 1;
        afficherCouleur( valeurRouge, valeurVert, valeurBleu );
        delay( PAUSE );
    }
  
    valeurRouge = 0;
    valeurBleu  = 0;
    valeurVert  = 255;
    for( int i = 0 ; i < 255 ; i += 1 )
    {
        valeurBleu += 1;
        valeurVert -= 1;
        afficherCouleur( valeurRouge, valeurVert, valeurBleu );
        delay( PAUSE );
    }
  
    valeurRouge = 0;
    valeurBleu = 255;
    valeurVert = 0;
   
    for( int i = 0 ; i < 255 ; i += 1 )
    {
        valeurRouge += 1;
        valeurBleu  -= 1;
        afficherCouleur( valeurRouge, valeurVert, valeurBleu );
        delay( PAUSE );
    }
    /* ******************** FIN PARTIE 1 ****************** */
#endif
#ifdef __PARTIE_2
    /* ******************* DEBUT PARTIE 2 ***************** */
    int lecture_echo_0, lecture_echo_1; 
    int distance_0, distance_1;

    digitalWrite(trigPIN0, LOW); 
    delayMicroseconds(2);                       // duree de 2us de l'onde sonore emise
    digitalWrite(trigPIN0, HIGH); 
    delayMicroseconds(10);                      // duree de 10us de l'onde sonore emise
    digitalWrite(trigPIN0, LOW); 
    lecture_echo_0 = pulseIn(echoPIN0, HIGH);   // mesure la duree de trajet aller-retour l'onde sonore 

    digitalWrite(trigPIN1, LOW); 
    delayMicroseconds(2);                       // duree de 2us de l'onde sonore emise
    digitalWrite(trigPIN1, HIGH); 
    delayMicroseconds(10);                      // duree de 10us de l'onde sonore emise
    digitalWrite(trigPIN1, LOW); 
    lecture_echo_1 = pulseIn(echoPIN1, HIGH);   // mesure la duree de trajet aller-retour l'onde sonore 

    distance_0 = lecture_echo_0 / 58.2; 
    distance_1 = lecture_echo_1 / 58.2; 
    //distance_0 = (int) ((float)lecture_echo_0 * coeff_multiplicateur);     // en cm
    //distance_1 = (int) ((float)lecture_echo_1 * coeff_multiplicateur);     // en cm

    if ( distance_0 >= maxDistance || distance_0 <= minDistance )
    {
        __println("Mesure_0 erronee, objet hors portee ou perdue ou indetectable !");
        distance_0 = maxDistance;
    }
    else
    {
        __print("Distance 0(cm) : "); 
        __println(distance_0);   // affiche la distance mesuree
    }
    if ( distance_1 >= maxDistance || distance_1 <= minDistance )
    {
        __println("Mesure_1 erronee, objet hors portee ou perdue ou indetectable !");
        distance_1 = maxDistance;
    }
    else
    {
        __print("Distance 1(cm) : "); 
        __println(distance_1);   // affiche la distance mesuree
    }

    // Modulation du cycle LED selon la distance du l'objet
    // (Définition de cycle long ou duree longue - cycle court ou duree courte)
    valeurRouge = 255; valeurBleu = 0; valeurVert = 0;
    afficherCouleur( valeurRouge, valeurVert, valeurBleu );
    delay(distance_0 * PAUSE);
    valeurRouge = 0; valeurBleu = 0; valeurVert = 0;
    afficherCouleur( valeurRouge, valeurVert, valeurBleu );
    delay(distance_0 * PAUSE);
    
    /* ******************** FIN PARTIE 2 ****************** */
#endif
#ifdef __PARTIE_3
    /* ******************* DEBUT PARTIE 3 ***************** */
    int lecture_echo_0, lecture_echo_1; 
    int distance_0, distance_1;

    digitalWrite(trigPIN0, LOW); 
    delayMicroseconds(2);                       // duree de 2us de l'onde sonore emise
    digitalWrite(trigPIN0, HIGH); 
    delayMicroseconds(10);                      // duree de 10us de l'onde sonore emise
    digitalWrite(trigPIN0, LOW); 
    lecture_echo_0 = pulseIn(echoPIN0, HIGH);   // mesure la duree de trajet aller-retour l'onde sonore 

    digitalWrite(trigPIN1, LOW); 
    delayMicroseconds(2);                       // duree de 2us de l'onde sonore emise
    digitalWrite(trigPIN1, HIGH); 
    delayMicroseconds(10);                      // duree de 10us de l'onde sonore emise
    digitalWrite(trigPIN1, LOW); 
    lecture_echo_1 = pulseIn(echoPIN1, HIGH);   // mesure la duree de trajet aller-retour l'onde sonore 

    distance_0 = lecture_echo_0 / 58.2; 
    distance_1 = lecture_echo_1 / 58.2; 
    //distance_0 = (int) ((float)lecture_echo_0 * coeff_multiplicateur);     // en cm
    //distance_1 = (int) ((float)lecture_echo_1 * coeff_multiplicateur);     // en cm

    if ( distance_0 >= maxDistance || distance_0 <= minDistance )
    {
        __println("Mesure_0 erronee, objet hors portee ou perdue ou indetectable !");
        distance_0 = maxDistance;
    }
    else
    {
        __print("Distance 0(cm) : "); 
        __println(distance_0);   // affiche la distance mesuree
    }
    if ( distance_1 >= maxDistance || distance_1 <= minDistance )
    {
        __println("Mesure_1 erronee, objet hors portee ou perdue ou indetectable !");
        distance_1 = maxDistance;
    }
    else
    {
        __print("Distance 1(cm) : "); 
        __println(distance_1);   // affiche la distance mesuree
    }

    // Animation par modulation du cycle et de la couleur de la LED selon la distance du l'objet
    animeLampe(distance_0);
  
    /* ******************** FIN PARTIE 3 ****************** */
#endif
#ifdef __PARTIE_4
    /* ******************* DEBUT PARTIE 4 ***************** */
    while( etat_courant != STATE_MAX )
    {
        Automate( lectureCmdUtilisateur() );
        delay( PAUSE );
    }
    
    etat_courant = STATE_MAX;
    
    __println("FIN D'EXECUTION");   // Fin d'exéction
    delay( PAUSE * 1000 );
    /* ******************** FIN PARTIE 4 ****************** */
#endif
#ifdef __PARTIE_5
    /* ******************* DEBUT PARTIE 5 ***************** */
    __println("A FAIRE");   // A FAIRE
    delay( PAUSE * 1000 );
    /* ******************** FIN PARTIE 5 ****************** */
#endif
}
